from okx.okxclient import OkxClient
from okx.consts import *


class FlexibleLoanAPI(OkxClient):
    def __init__(self, api_key='-1', api_secret_key='-1', passphrase='-1', use_server_time=None, flag='1',
                 domain='https://www.okx.com', debug=False, proxy=None):
        OkxClient.__init__(self, api_key, api_secret_key, passphrase, use_server_time, flag, domain, debug, proxy)

    def borrow_currencies(self):
        return self._request_without_params(GET, FINANCE_BORROW_CURRENCIES)

    def collateral_assets(self, ccy=''):
        params = {}
        if ccy != '':
            params['ccy'] = ccy
        return self._request_with_params(GET, FINANCE_COLLATERAL_ASSETS, params)

    def max_loan(self, borrowCcy='', supCollateral=[]):
        params = {
            'borrowCcy': borrowCcy,
            'supCollateral': supCollateral,
        }
        return self._request_with_params(POST, FINANCE_MAX_LOAN, params)

    def max_collateral_redeem_amount(self, ccy=''):
        params = {}
        if ccy != '':
            params['ccy'] = ccy
        return self._request_with_params(GET, FINANCE_MAX_REDEEM, params)

    def adjust_collateral(self, type='', collateralCcy='', collateralAmt=''):
        params = {
            'type': type,
            'collateralCcy': collateralCcy,
            'collateralAmt': collateralAmt,
        }
        return self._request_with_params(POST, FINANCE_ADJUST_COLLATERAL, params)

    def loan_info(self):
        return self._request_without_params(GET, FINANCE_LOAN_INFO)

    def loan_history(self, type='', after='', before='', limit=''):
        params = {}
        if type != '':
            params['type'] = type
        if after != '':
            params['after'] = after
        if before != '':
            params['before'] = before
        if limit != '':
            params['limit'] = limit
        return self._request_with_params(GET, FINANCE_LOAN_HISTORY, params)

    def interest_accrued(self, ccy='', after='', before='', limit=''):
        params = {}
        if ccy != '':
            params['ccy'] = ccy
        if after != '':
            params['after'] = after
        if before != '':
            params['before'] = before
        if limit != '':
            params['limit'] = limit
        return self._request_with_params(GET, FINANCE_INTEREST_ACCRUED, params)



from .constants import *
from .restapi import *
from .wsapi import *


class OkxRestClient:

    def __init__(
        self,
        apikey="",
        apisecret="",
        passphrase="",
        use_server_time=False,
        simulation=False,
        domain=API_URL,
        debug=False,
        proxy=None,
    ):

        self.account = AccountClient(
            apikey,
            apisecret,
            passphrase,
            use_server_time,
            simulation,
            domain,
            debug,
            proxy,
        )
        self.funding = FundingClient(
            apikey,
            apisecret,
            passphrase,
            use_server_time,
            simulation,
            domain,
            debug,
            proxy,
        )
        self.subaccount = SubAccountClient(
            apikey,
            apisecret,
            passphrase,
            use_server_time,
            simulation,
            domain,
            debug,
            proxy,
        )

        self.public = PublicDataClient(
            apikey,
            apisecret,
            passphrase,
            use_server_time,
            simulation,
            domain,
            debug,
            proxy,
        )
        self.market = PublicDataClient(
            apikey,
            apisecret,
            passphrase,
            use_server_time,
            simulation,
            domain,
            debug,
            proxy,
        )

        self.trade = TradingClient(
            apikey,
            apisecret,
            passphrase,
            use_server_time,
            simulation,
            domain,
            debug,
            proxy,
        )
        self.algotrade = AlgoTradingClient(
            apikey,
            apisecret,
            passphrase,
            use_server_time,
            simulation,
            domain,
            debug,
            proxy,
        )
        self.blocktrade = BlockTradingClient(
            apikey,
            apisecret,
            passphrase,
            use_server_time,
            simulation,
            domain,
            debug,
            proxy,
        )
        self.copytrade = CopyTradingClient(
            apikey,
            apisecret,
            passphrase,
            use_server_time,
            simulation,
            domain,
            debug,
            proxy,
        )
        self.gridtrade = GridTradingClient(
            apikey,
            apisecret,
            passphrase,
            use_server_time,
            simulation,
            domain,
            debug,
            proxy,
        )
        self.recurringbuy = RecurringBuyClient(
            apikey,
            apisecret,
            passphrase,
            use_server_time,
            simulation,
            domain,
            debug,
            proxy,
        )
        self.signaltrade = SignalTradingClient(
            apikey,
            apisecret,
            passphrase,
            use_server_time,
            simulation,
            domain,
            debug,
            proxy,
        )
        self.spreadtrade = SpreadTradingClient(
            apikey,
            apisecret,
            passphrase,
            use_server_time,
            simulation,
            domain,
            debug,
            proxy,
        )

        self.finance = FinanceClient(
            apikey,
            apisecret,
            passphrase,
            use_server_time,
            simulation,
            domain,
            debug,
            proxy,
        )
        self.rubik = RubikClient(
            apikey,
            apisecret,
            passphrase,
            use_server_time,
            simulation,
            domain,
            debug,
            proxy,
        )

        self.ndbroker = NonDisclosedBrokerClient(
            apikey,
            apisecret,
            passphrase,
            use_server_time,
            simulation,
            domain,
            debug,
            proxy,
        )
        self.fdbroker = FullyDisclosedBrokerClient(
            apikey,
            apisecret,
            passphrase,
            use_server_time,
            simulation,
            domain,
            debug,
            proxy,
        )


class OkxSocketClient:

    def __init__(
        self,
        apikey="",
        apisecret="",
        passphrase="",
        public_url=WS_URL_PUBLIC,
        private_url=WS_URL_PRIVATE,
        business_url=WS_URL_BUSINESS,
        use_server_time=False,
    ):
        self.public = PublicAsyncClient(public_url)
        self.private = PrivateAsyncClient(
            apikey, apisecret, passphrase, private_url, use_server_time
        )
        self.business = PrivateAsyncClient(
            apikey, apisecret, passphrase, business_url, use_server_time
        )



from .BaseClient import OkxBaseClient
from ..constants import *


class NonDisclosedBrokerClient(OkxBaseClient):
    def __init__(
        self,
        apikey="",
        apisecret="",
        passphrase="",
        use_server_time=False,
        simulation=False,
        domain=API_URL,
        debug=False,
        proxy=None,
    ):
        OkxBaseClient.__init__(
            self,
            apikey,
            apisecret,
            passphrase,
            use_server_time,
            simulation,
            domain,
            debug,
            proxy,
        )

    def get_broker_info(self):
        return self._request(GET, BROKER_ND_INFO)

    def create_subaccount(self, subAcct="", label=""):
        params = {"subAcct": subAcct, "label": label}
        return self._request(POST, BROKER_ND_CREATE_SUBACCOUNT, params)

    def delete_subaccount(self, subAcct=""):
        params = {"subAcct": subAcct}
        return self._request(POST, BROKER_ND_DELETE_SUBACCOUNT, params)

    def get_subaccount_info(self, subAcct="", page="", limit=""):
        params = {"subAcct": subAcct, "page": page, "limit": limit}
        return self._request(GET, BROKER_ND_SUBACCOUNT_INFO, params)

    def create_subaccount_apikey(
        self, subAcct="", label="", passphrase="", ip="", perm=""
    ):
        params = {
            "subAcct": subAcct,
            "label": label,
            "passphrase": passphrase,
            "ip": ip,
            "perm": perm,
        }
        return self._request(POST, BROKER_ND_CREATE_APIKEY, params)

    def get_subaccount_apikey(self, subAcct="", apiKey=""):
        params = {"subAcct": subAcct, "apiKey": apiKey}
        return self._request(GET, BROKER_ND_SELECT_APIKEY, params)

    def reset_subaccount_apikey(self, subAcct="", apiKey="", label="", perm="", ip=""):
        params = {
            "subAcct": subAcct,
            "apiKey": apiKey,
            "label": label,
            "perm": perm,
            "ip": ip,
        }
        return self._request(POST, BROKER_ND_MODIFY_APIKEY, params)

    def delete_subaccount_apikey(self, subAcct="", apiKey=""):
        params = {"subAcct": subAcct, "apiKey": apiKey}
        return self._request(POST, BROKER_ND_DELETE_APIKEY, params)

    def set_subaccount_level(self, subAcct="", acctLv=""):
        params = {"subAcct": subAcct, "acctLv": acctLv}
        return self._request(POST, BROKER_ND_SET_SUBACCOUNT_LEVEL, params)

    def set_subaccount_fee_rate(
        self, subAcct="", instType="", chgType="", chgTaker="", chgMaker="", effDate=""
    ):
        params = {
            "subAcct": subAcct,
            "instType": instType,
            "chgType": chgType,
            "chgTaker": chgTaker,
            "chgMaker": chgMaker,
            "effDate": effDate,
        }
        return self._request(POST, BROKER_ND_SET_SUBACCOUNT_FEE_RATE, params)

    def create_subaccount_deposit_address(
        self, subAcct="", ccy="", chain="", addrType="", to=""
    ):
        params = {
            "subAcct": subAcct,
            "ccy": ccy,
            "chain": chain,
            "addrType": addrType,
            "to": to,
        }
        return self._request(POST, BROKER_ND_SUBACCOUNT_DEPOSIT_ADDRESS, params)

    def reset_subaccount_deposit_address(
        self, subAcct="", ccy="", chain="", addr="", to=""
    ):
        params = {
            "subAcct": subAcct,
            "ccy": ccy,
            "chain": chain,
            "addr": addr,
            "to": to,
        }
        return self._request(POST, BROKER_ND_MODIFY_SUBACCOUNT_DEPOSIT_ADDRESS, params)

    def get_subaccount_deposit_address(self, subAcct="", ccy=""):
        params = {"subAcct": subAcct, "ccy": ccy}
        return self._request(GET, BROKER_ND_SUBACCOUNT_DEPOSIT_ADDRESS, params)

    def get_subaccount_deposit_history(
        self, subAcct="", ccy="", txId="", state="", after="", before="", limit=""
    ):
        params = {
            "subAcct": subAcct,
            "ccy": ccy,
            "txId": txId,
            "state": state,
            "after": after,
            "before": before,
            "limit": limit,
        }
        return self._request(GET, BROKER_ND_SUBACCOUNT_DEPOSIT_HISTORY, params)

    def get_subaccount_withdrawal_history(
        self,
        subAcct="",
        ccy="",
        wdId="",
        clientId="",
        txId="",
        type="",
        state="",
        before="",
        limit="",
    ):
        params = {
            "subAcct": subAcct,
            "ccy": ccy,
            "wdId": wdId,
            "clientId": clientId,
            "txId": txId,
            "type": type,
            "state": state,
            "before": before,
            "limit": limit,
        }
        return self._request(GET, BROKER_ND_SUBACCOUNT_WITHDRAWAL_HISTORY, params)

    def get_rebate_daily(self, subAcct="", begin="", end="", page="", limit=""):
        params = {
            "subAcct": subAcct,
            "begin": begin,
            "end": end,
            "page": page,
            "limit": limit,
        }
        return self._request(GET, BROKER_ND_REBATE_DAILY, params)

    def get_rebate_details_download_link(self, type="", begin="", end=""):
        params = {"type": type, "begin": begin, "end": end}
        return self._request(GET, BROKER_ND_GET_REBATE_PER_ORDERS, params)

    def generate_rebate_details_download_link(self, begin="", end=""):
        params = {"begin": begin, "end": end}
        return self._request(POST, BROKER_ND_GEN_REBATE_PER_ORDERS, params)

    def get_dcd_products(self, ccy, alternativeCcy, optType, tag):
        params = {
            "ccy": ccy,
            "alternativeCcy": alternativeCcy,
            "optType": optType,
            "tag": tag,
        }
        return self._request(GET, FINANCE_SFP_DCD_PRODUCTS, params)

    def request_dcd_quote(
        self, notional, notionalCcy, productId, tag, markUp="", clReqId=""
    ):
        params = {
            "notional": notional,
            "notionalCcy": notionalCcy,
            "productId": productId,
            "tag": tag,
            "markUp": markUp,
            "clReqId": clReqId,
        }
        return self._request(POST, FINANCE_SFP_DCD_QUOTE, params)

    def exec_dcd_order(self, quoteId, clReqId=""):
        params = {"quoteId": quoteId, "clReqId": clReqId}
        return self._request(GET, FINANCE_SFP_DCD_ORDER, params)

    def get_dcd_order(self, ordId="", clReqId=""):
        params = {"ordId": ordId, "clReqId": clReqId}
        return self._request(GET, FINANCE_SFP_DCD_ORDER, params)

    def get_dcd_orders(
        self,
        productId="",
        uly="",
        state="",
        beginId="",
        endId="",
        begin="",
        end="",
        limit="",
    ):
        params = {
            "productId": productId,
            "uly": uly,
            "state": state,
            "beginId": beginId,
            "endId": endId,
            "begin": begin,
            "end": end,
            "limit": limit,
        }
        return self._request(GET, FINANCE_SFP_DCD_ORDERS, params)

    def set_subaccount_asset(self, subAcct, ccy):
        params = {"subAcct": subAcct, "ccy": ccy}
        return self._request(POST, BROKER_ND_SET_SUBACCOUNT_ASSETS, params)

    def report_subaccount_ip(self, subAcct, clientIP):
        params = {"subAcct": subAcct, "clientIP": clientIP}
        return self._request(POST, BROKER_ND_REPORT_SUBACCOUNT_IP, params)

    def get_rebate_info(self, apiKey="", uid="", subAcct=""):
        params = {"apiKey": apiKey, "uid": uid, "subAcct": subAcct}
        return self._request(GET, BROKER_ND_IF_REBATE, params)


class FullyDisclosedBrokerClient(OkxBaseClient):
    def __init__(
        self,
        apikey="",
        apisecret="",
        passphrase="",
        use_server_time=False,
        simulation=False,
        domain=API_URL,
        debug=False,
        proxy=None,
    ):
        OkxBaseClient.__init__(
            self,
            apikey,
            apisecret,
            passphrase,
            use_server_time,
            simulation,
            domain,
            debug,
            proxy,
        )

    def get_rebate_details_download_link(self, type="", begin="", end=""):
        params = {"type": type, "begin": begin, "end": end}
        return self._request(GET, BROKER_FD_GET_REBATE_PER_ORDERS, params)

    def generate_rebate_details_download_link(self, begin="", end=""):
        params = {"begin": begin, "end": end}
        return self._request(POST, BROKER_FD_GEN_REBATE_PER_ORDERS, params)

    def get_users_broker_rebate_information(self, apiKey, brokerType):
        params = {"apiKey": apiKey, "brokerType": brokerType}
        return self._request(GET, BROKER_FD_IF_REBATE, params)



from .BaseClient import OkxBaseClient
from ..constants import *


class GridTradingClient(OkxBaseClient):
    def __init__(
        self,
        apikey: str = "",
        apisecret: str = "",
        passphrase: str = "",
        use_server_time: bool = False,
        simulation: bool = False,
        domain: str = API_URL,
        debug: bool = False,
        proxy: dict | None = None,
    ):
        OkxBaseClient.__init__(
            self,
            apikey,
            apisecret,
            passphrase,
            use_server_time,
            simulation,
            domain,
            debug,
            proxy,
        )

    def place_order(
        self,
        instId: str,
        algoOrdType: str,
        maxPx: float,
        minPx: float,
        gridNum: int,
        algoClOrdId: str | None = None,
        runType: str = "",
        tpTriggerPx: str = "",
        slTriggerPx: str = "",
        tag: str = BROKER_ID,
        profitSharingRatio: float | None = None,
        triggerParams: dict | None = None,
        quoteSz: str = "",
        baseSz: str = "",
        sz: str = "",
        direction: str = "",
        lever: str = "",
        basePos: str = "",
        tpRatio: str = "",
        slRatio: str = "",
    ) -> dict:
        params = {
            "algoClOrdId": algoClOrdId,
            "instId": instId,
            "algoOrdType": algoOrdType,
            "maxPx": maxPx,
            "minPx": minPx,
            "gridNum": gridNum,
            "runType": runType,
            "tpTriggerPx": tpTriggerPx,
            "slTriggerPx": slTriggerPx,
            "tag": tag,
            "profitSharingRatio": profitSharingRatio,
            "triggerParams": triggerParams,
            "quoteSz": quoteSz,
            "baseSz": baseSz,
            "sz": sz,
            "direction": direction,
            "lever": lever,
            "basePos": basePos,
            "tpRatio": tpRatio,
            "slRatio": slRatio,
        }
        return self._request(POST, GRID_ORDER_ALGO, params)

    def amend_order(
        self, algoId: str, instId: str, slTriggerPx: str = "", tpTriggerPx: str = ""
    ) -> dict:
        params = {
            "algoId": algoId,
            "instId": instId,
            "slTriggerPx": slTriggerPx,
            "tpTriggerPx": tpTriggerPx,
        }
        return self._request(POST, GRID_AMEND_ORDER_ALGO, params)

    def stop_order(
        self, algoId: str, instId: str, algoOrdType: str, stopType: str
    ) -> dict:
        params = [
            {
                "algoId": algoId,
                "instId": instId,
                "algoOrdType": algoOrdType,
                "stopType": stopType,
            }
        ]
        return self._request(POST, GRID_STOP_ORDER_ALGO, params)

    def close_position(
        self, algoId: str, mktClose: bool, sz: str = "", px: str = ""
    ) -> dict:
        params = [{"algoId": algoId, "mktClose": mktClose, "sz": sz, "px": px}]
        return self._request(POST, GRID_CLOSE_POSITION, params)

    def cancel_close_position_order(self, algoId: str, ordId: str) -> dict:
        params = [{"algoId": algoId, "ordId": ordId}]
        return self._request(POST, GRID_CANCEL_CLOSE_ORDER, params)

    def get_pending_orders(
        self,
        algoOrdType: str = "",
        algoId: str = "",
        instId: str = "",
        instType: str = "",
        after: str = "",
        before: str = "",
        limit: str = "",
        instFamily: str = "",
    ) -> dict:
        params = {
            "algoOrdType": algoOrdType,
            "algoId": algoId,
            "instId": instId,
            "instType": instType,
            "after": after,
            "before": before,
            "limit": limit,
            "instFamily": instFamily,
        }
        return self._request(GET, GRID_ORDERS_ALGO_PENDING, params)

    def get_orders_history(
        self,
        algoOrdType: str = "",
        algoId: str = "",
        instId: str = "",
        instType: str = "",
        after: str = "",
        before: str = "",
        limit: str = "",
        instFamily: str = "",
    ) -> dict:
        params = {
            "algoOrdType": algoOrdType,
            "algoId": algoId,
            "instId": instId,
            "instType": instType,
            "after": after,
            "before": before,
            "limit": limit,
            "instFamily": instFamily,
        }
        return self._request(GET, GRID_ORDERS_ALGO_HISTORY, params)

    def get_orders_details(self, algoOrdType: str = "", algoId: str = "") -> dict:
        params = {"algoOrdType": algoOrdType, "algoId": algoId}
        return self._request(GET, GRID_ORDERS_ALGO_DETAILS, params)

    def get_sub_orders(
        self,
        algoId: str = "",
        algoOrdType: str = "",
        type: str = "",
        groupId: str = "",
        after: str = "",
        before: str = "",
        limit: str = "",
    ) -> dict:
        params = {
            "algoId": algoId,
            "algoOrdType": algoOrdType,
            "type": type,
            "groupId": groupId,
            "after": after,
            "before": before,
            "limit": limit,
        }
        return self._request(GET, GRID_SUB_ORDERS, params)

    def get_positions(self, algoOrdType: str = "", algoId: str = "") -> dict:
        params = {"algoOrdType": algoOrdType, "algoId": algoId}
        return self._request(GET, GRID_POSITIONS, params)

    def withdraw_income(self, algoId: str = "") -> dict:
        params = {"algoId": algoId}
        return self._request(POST, GRID_WITHDRAW_INCOME, params)

    def compute_margin_balance(
        self, algoId: str = "", type: str = "", amt: str = ""
    ) -> dict:
        params = {"algoId": algoId, "type": type, "amt": amt}
        return self._request(POST, GRID_COMPUTE_MARGIN_BALANCE, params)

    def adjust_margin_balance(
        self, algoId: str = "", type: str = "", amt: str = "", percent: str = ""
    ) -> dict:
        params = {"algoId": algoId, "type": type, "amt": amt, "percent": percent}
        return self._request(POST, GRID_MARGIN_BALANCE, params)

    def get_ai_param(
        self,
        algoOrdType: str = "",
        instId: str = "",
        direction: str = "",
        duration: str = "",
    ) -> dict:
        params = {
            "algoOrdType": algoOrdType,
            "instId": instId,
            "direction": direction,
            "duration": duration,
        }
        return self._request(GET, GRID_AI_PARAM, params)

    def compute_min_investment(
        self,
        instId: str,
        algoOrdType: str,
        maxPx: float,
        minPx: float,
        gridNum: int,
        runType: str,
        direction: str = "",
        lever: str = "",
        basePos: str = "",
        investmentData: list = [],
    ) -> dict:
        params = {
            "instId": instId,
            "algoOrdType": algoOrdType,
            "maxPx": maxPx,
            "minPx": minPx,
            "gridNum": gridNum,
            "runType": runType,
            "direction": direction,
            "lever": lever,
            "basePos": basePos,
            "investmentData": investmentData,
        }
        return self._request(POST, GRID_MIN_INVESTMENT, params)

    def get_rsi_back_testing(
        self,
        instId: str,
        timeframe: str,
        thold: float,
        timePeriod: int,
        triggerCond: str = "",
        duration: str = "",
    ) -> dict:
        params = {
            "instId": instId,
            "timeframe": timeframe,
            "thold": thold,
            "timePeriod": timePeriod,
            "triggerCond": triggerCond,
            "duration": duration,
        }
        return self._request(GET, GRID_AI_PARAM, params)



from okx.okxclient import OkxClient
from okx.consts import *


class SolStakingAPI(OkxClient):
    def __init__(self, api_key='-1', api_secret_key='-1', passphrase='-1', use_server_time=None, flag='1', domain = 'https://www.okx.com',debug = False, proxy=None):
        OkxClient.__init__(self, api_key, api_secret_key, passphrase, use_server_time, flag, domain, debug, proxy)

    def sol_purchase(self, amt):

        params = {
            'amt': amt,
        }
        return self._request_with_params(POST, STACK_SOL_PURCHASE, params)

    def sol_redeem(self, amt=''):

        params = {
            'amt': amt,
        }
        return self._request_with_params(POST, STACK_SOL_REDEEM, params)

    def sol_balance(self):

        params = {}
        return self._request_with_params(GET, STACK_SOL_BALANCE, params)

    def sol_purchase_redeem_history(self, type='', status='', after='', before='', limit=''):

        params = {}
        if type != '':
            params['type'] = type
        if status != '':
            params['status'] = status
        if after != '':
            params['after'] = after
        if before != '':
            params['before'] = before
        if limit != '':
            params['limit'] = limit
        return self._request_with_params(GET, STACK_SOL_PURCHASE_REDEEM_HISTORY, params)

    def sol_apy_history(self, days):

        params = {
            'days': days,
        }
        return self._request_with_params(GET, STACK_SOL_APY_HISTORY, params)

    def sol_product_info(self):
        return self._request_without_params(GET, STACK_SOL_PRODUCT_INFO)



from .BaseClient import OkxBaseClient
from ..constants import *


class BlockTradingClient(OkxBaseClient):
    def __init__(
        self,
        apikey="",
        apisecret="",
        passphrase="",
        use_server_time=False,
        simulation=False,
        domain=API_URL,
        debug=False,
        proxy=None,
    ):
        OkxBaseClient.__init__(
            self,
            apikey,
            apisecret,
            passphrase,
            use_server_time,
            simulation,
            domain,
            debug,
            proxy,
        )

    def counterparties(self):
        params = {}
        return self._request(GET, RFQ_COUNTERPARTIES, params)

    def create_rfq(
        self,
        counterparties=[],
        anonymous="false",
        clRfqId="",
        allowPartialExecution="false",
        legs=[],
    ):
        params = {
            "counterparties": counterparties,
            "anonymous": anonymous,
            "clRfqId": clRfqId,
            "tag": BROKER_ID,
            "allowPartialExecution": allowPartialExecution,
            "legs": legs,
        }
        return self._request(POST, RFQ_CREATE_RFQ, params)

    def cancel_rfq(self, rfqId="", clRfqId=""):
        params = {"rfqId": rfqId, "clRfqId": clRfqId}
        return self._request(POST, RFQ_CANCEL_RFQ, params)

    def cancel_batch_rfqs(self, rfqIds=[], clRfqIds=[]):
        params = {"rfqIds": rfqIds, "clRfqIds": clRfqIds}
        return self._request(POST, RFQ_CANCEL_BATCH_RFQS, params)

    def cancel_all_rfqs(self):
        params = {}
        return self._request(POST, RFQ_CANCEL_ALL_RSQS, params)

    def execute_quote(self, rfqId="", quoteId="", legs=[]):
        params = {"rfqId": rfqId, "quoteId": quoteId, "legs": legs}
        return self._request(POST, RFQ_EXECUTE_QUOTE, params)

    # Get Quote products
    def get_quote_products(self):
        return self._request(GET, RFQ_GET_MARKER_INSTRUMENT_SETTING)

    # Set Quote products
    def set_marker_instrument(self, params=[]):
        return self._request(POST, RFQ_SET_MARKER_INSTRUMENT_SETTING, params)

    def reset_mmp(self):
        return self._request(POST, RFQ_MMP_RESET)

    def set_mmp_config(self, timeInterval, frozenInterval, countLimit):
        params = {
            "timeInterval": timeInterval,
            "frozenInterval": frozenInterval,
            "countLimit": countLimit,
        }
        return self._request(POST, RFQ_SET_MMP_CONFIG, params)

    def get_mmp_config(
        self,
        timeInterval="",
        frozenInterval="",
        countLimit="",
        mmpFrozen="",
        mmpFrozenUntil="",
    ):
        params = {
            "timeInterval": timeInterval,
            "frozenInterval": frozenInterval,
            "countLimit": countLimit,
            "mmpFrozen": mmpFrozen,
            "mmpFrozenUntil": mmpFrozenUntil,
        }
        return self._request(GET, RFQ_GET_MMP_CONFIG, params)

    def create_quote(
        self,
        rfqId="",
        clQuoteId="",
        quoteSide="",
        legs=[],
        anonymous=False,
        expiresIn="",
    ):
        params = {
            "rfqId": rfqId,
            "clQuoteId": clQuoteId,
            "tag": BROKER_ID,
            "quoteSide": quoteSide,
            "legs": legs,
            "anonymous": anonymous,
            "expiresIn": expiresIn,
        }
        return self._request(POST, RFQ_CREATE_QUOTE, params)

    def cancel_quote(self, quoteId="", clQuoteId=""):
        params = {"quoteId": quoteId, "clQuoteId": clQuoteId}
        return self._request(POST, RFQ_CANCEL_QUOTE, params)

    def cancel_batch_quotes(self, quoteIds="", clQuoteIds=""):
        params = {"quoteIds": quoteIds, "clQuoteIds": clQuoteIds}
        return self._request(POST, RFQ_CANCEL_BATCH_QUOTES, params)

    def cancel_all_quotes(self):
        params = {}
        return self._request(POST, RFQ_CANCEL_ALL_QUOTES, params)

    def get_rfqs(self, rfqId="", clRfqId="", state="", beginId="", endId="", limit=""):
        params = {
            "rfqId": rfqId,
            "clRfqId": clRfqId,
            "state": state,
            "beginId": beginId,
            "endId": endId,
            "limit": limit,
        }
        return self._request(GET, RFQ_RFQS, params)

    def get_quotes(
        self,
        rfqId="",
        clRfqId="",
        quoteId="",
        clQuoteId="",
        state="",
        beginId="",
        endId="",
        limit="",
    ):
        params = {
            "rfqId": rfqId,
            "clRfqId": clRfqId,
            "quoteId": quoteId,
            "clQuoteId": clQuoteId,
            "state": state,
            "beginId": beginId,
            "endId": endId,
            "limit": limit,
        }
        return self._request(GET, RFQ_QUOTES, params)

    def get_trades(
        self,
        rfqId="",
        clRfqId="",
        quoteId="",
        clQuoteId="",
        state="",
        beginId="",
        endId="",
        beginTs="",
        endTs="",
        limit="",
    ):
        params = {
            "rfqId": rfqId,
            "clRfqId": clRfqId,
            "quoteId": quoteId,
            "clQuoteId": clQuoteId,
            "state": state,
            "beginId": beginId,
            "endId": endId,
            "beginTs": beginTs,
            "endTs": endTs,
            "limit": limit,
        }
        return self._request(GET, RFQ_TRADES, params)

    def get_public_trades(self, beginId="", endId="", limit=""):
        params = {"beginId": beginId, "endId": endId, "limit": limit}
        return self._request(GET, RFQ_PUBLIC_TRADES, params)

    # Get block tickers
    def get_block_tickers(self, instType="", uly="", instFamily=""):
        params = {"instType": instType, "uly": uly, "instFamily": instFamily}
        return self._request(GET, MARKET_BLOCK_TICKERS, params)

    # Get block ticker
    def get_block_ticker(self, instId=""):
        params = {"instId": instId}
        return self._request(GET, MARKET_BLOCK_TICKER, params)

    # Get block trades
    def get_block_trades(self, instId=""):
        params = {"instId": instId}
        return self._request(GET, MARKET_BLOCK_TRADES, params)



from .Account import AccountClient
from .AlgoTrading import AlgoTradingClient
from .BlockTrading import BlockTradingClient
from .Broker import NonDisclosedBrokerClient
from .Broker import FullyDisclosedBrokerClient
from .CopyTrading import CopyTradingClient
from .Finance import FinanceClient
from .Funding import FundingClient
from .GridTrading import GridTradingClient
from .PublicData import PublicDataClient
from .RecurringBuy import RecurringBuyClient
from .Rubik import RubikClient
from .SignalTrading import SignalTradingClient
from .SpreadTrading import SpreadTradingClient
from .SubAccount import SubAccountClient
from .Trading import TradingClient

__all__ = [
    "AccountClient",
    "AlgoTradingClient",
    "BlockTradingClient",
    "NonDisclosedBrokerClient",
    "FullyDisclosedBrokerClient",
    "CopyTradingClient",
    "FinanceClient",
    "FundingClient",
    "GridTradingClient",
    "PublicDataClient",
    "RecurringBuyClient",
    "RubikClient",
    "SignalTradingClient",
    "SpreadTradingClient",
    "SubAccountClient",
    "TradingClient",
]



from .okxclient import OkxClient
from .consts import *


class CopyTradingAPI(OkxClient):
    def __init__(self, api_key='-1', api_secret_key='-1', passphrase='-1', use_server_time=False, flag='1',
                 domain='https://www.okx.com', debug=False, proxy=None):
        OkxClient.__init__(self, api_key, api_secret_key, passphrase, use_server_time, flag, domain, debug,
                           proxy=proxy)

    # Get existing leading positions
    def get_existing_leading_positions(self, instId=''):
        params = {
            'instId': instId
        }
        return self._request_with_params(GET, GET_EXISTING_LEADING_POSITIONS, params)

    # Get leading position history
    def get_leading_position_history(self, instId='', after='', before='', limit=''):
        params = {
            'instId': instId,
            'after': after,
            'before': before,
            'limit': limit
        }
        return self._request_with_params(GET, GET_LEADING_POSITIONS_HISTORY, params)

    # Place leading stop order
    def place_leading_stop_order(self, subPosId='', tpTriggerPx='', slTriggerPx='', tpTriggerPxType='',
                                 slTriggerPxType=''):
        params = {
            'subPosId': subPosId,
            'tpTriggerPx': tpTriggerPx,
            'slTriggerPx': slTriggerPx,
            'tpTriggerPxType': tpTriggerPxType,
            'slTriggerPxType': slTriggerPxType
        }
        return self._request_with_params(POST, PLACE_LEADING_STOP_ORDER, params)

    # Close leading position
    def close_leading_position(self, subPosId=''):
        params = {
            'subPosId': subPosId
        }
        return self._request_with_params(POST, CLOSE_LEADING_POSITIONS, params)

    # Get leading instruments
    def get_leading_instruments(self):
        return self._request_without_params(GET, GET_LEADING_POSITIONS)

    # Amend leading instruments
    def amend_leading_instruments(self, instId=''):
        params = {
            'instId': instId
        }
        return self._request_with_params(POST, AMEND_EXISTING_LEADING_POSITIONS, params)

    # Get profit sharing details
    def get_profit_sharing_details(self, after='', before='', limit=''):
        params = {
            'after': after,
            'before': before,
            'limit': limit
        }
        return self._request_with_params(GET, GET_PROFIT_SHARING_DETAILS, params)

    # Get total profit sharing
    def get_total_profit_sharing(self):
        return self._request_without_params(GET, GET_TOTAL_PROFIT_SHARING)

    # Get unrealized profit sharing details
    def get_unrealized_profit_sharing_details(self):
        return self._request_without_params(GET, GET_UNREALIZED_PROFIT_SHARING_DETAILS)



from .BaseClient import OkxBaseClient
from ..constants import *


class RubikClient(OkxBaseClient):

    def __init__(
        self,
        apikey="",
        apisecret="",
        passphrase="",
        use_server_time=False,
        simulation=False,
        domain=API_URL,
        debug=False,
        proxy=None,
    ):
        OkxBaseClient.__init__(
            self,
            apikey,
            apisecret,
            passphrase,
            use_server_time,
            simulation,
            domain,
            debug,
            proxy,
        )

    def get_support_coin(self):
        return self._request(GET, RUBIK_STAT_TRADING_DATA_SUPPORT_COIN)

    def get_taker_volume(self, ccy, instType, begin="", end="", period=""):
        params = {
            "ccy": ccy,
            "instType": instType,
            "begin": begin,
            "end": end,
            "period": period,
        }
        return self._request(GET, RUBIK_STAT_TAKER_VOLUME, params)

    def get_margin_lending_ratio(self, ccy, begin="", end="", period=""):
        params = {"ccy": ccy, "begin": begin, "end": end, "period": period}
        return self._request(GET, RUBIK_STAT_MARGIN_LOAN_RATIO, params)

    def get_long_short_ratio(self, ccy, begin="", end="", period=""):
        params = {"ccy": ccy, "begin": begin, "end": end, "period": period}
        return self._request(GET, RUBIK_STAT_CONTRACTS_LONG_SHORT_RATIO, params)

    def get_contracts_interest_volume(self, ccy, begin="", end="", period=""):
        params = {"ccy": ccy, "begin": begin, "end": end, "period": period}
        return self._request(GET, RUBIK_STAT_CONTRACTS_INTEREST_VOLUME, params)

    def get_options_interest_volume(self, ccy, period=""):
        params = {"ccy": ccy, "period": period}
        return self._request(GET, RUBIK_STAT_OPTION_OPEN_INTEREST_VOLUME, params)

    def get_put_call_ratio(self, ccy, period=""):
        params = {"ccy": ccy, "period": period}
        return self._request(GET, RUBIK_STAT_OPTION_OPEN_INTEREST_RATIO, params)

    def get_interest_volume_expiry(self, ccy, period=""):
        params = {"ccy": ccy, "period": period}
        return self._request(GET, RUBIK_STAT_OPTION_OPEN_INTEREST_VOLUME_EXPIRY, params)

    def get_interest_volume_strike(self, ccy, expTime, period=""):
        params = {"ccy": ccy, "expTime": expTime, "period": period}
        return self._request(GET, RUBIK_STAT_OPTION_OPEN_INTEREST_VOLUME_STRIKE, params)

    def get_taker_block_volume(self, ccy, period=""):
        params = {"ccy": ccy, "period": period}
        return self._request(GET, RUBIK_STAT_OPTION_TAKER_BLOCK_VOLUME, params)



from .okxclient import OkxClient
from .consts import *


class SpreadTradingAPI(OkxClient):

    def __init__(self, api_key='-1', api_secret_key='-1', passphrase='-1', use_server_time=None, flag='1', domain = 'https://www.okx.com',debug = False, proxy=None):
        OkxClient.__init__(self, api_key, api_secret_key, passphrase, use_server_time, flag, domain, debug, proxy)

    # Place Order
    def place_order(self, sprdId='', clOrdId='', tag='', side='', ordType='', sz='', px=''):
        params = {'sprdId': sprdId, 'clOrdId': clOrdId, 'tag': tag, 'side': side, 'ordType': ordType, 'sz': sz,
                  'px': px}
        return self._request_with_params(POST, SPREAD_PLACE_ORDER, params)

    # Cancel Order
    def cancel_order(self,ordId='', clOrdId=''):
        params = {'ordId': ordId, 'clOrdId': clOrdId}
        return self._request_with_params(POST, SPREAD_CANCEL_ORDER, params)

    # Cancel All orders
    def cancel_all_orders(self, sprdId=''):
        params = {'sprdId': sprdId}
        return self._request_with_params(POST, SPREAD_CANCEL_ALL_ORDERS, params)

    # Get order details
    def get_order_details(self, ordId='', clOrdId=''):
        params = {'ordId': ordId, 'clOrdId': clOrdId}
        return self._request_with_params(GET, SPREAD_GET_ORDER_DETAILS, params)

    # Get active orders
    def get_active_orders(self, sprdId='', ordType='', state='', beginId='', endId='', limit=''):
        params = {'sprdId': sprdId, 'ordType': ordType, 'state': state, 'beginId': beginId, 'endId': endId, 'limit': limit}
        return self._request_with_params(GET, SPREAD_GET_ACTIVE_ORDERS, params)

    # Get orders (last 7 days)
    def get_orders(self, sprdId='', ordType='', state='', beginId='', endId='', begin='', end='', limit=''):
        params = {'sprdId': sprdId, 'ordType': ordType, 'state': state, 'beginId': beginId, 'endId': endId,
                  'begin': begin, 'end': end, 'limit': limit}
        return self._request_with_params(GET, SPREAD_GET_ORDERS, params)

    # Get trades (last 7 days)
    def get_trades(self, sprdId='', tradeId='', ordId='', beginId='', endId='', begin='', end='', limit=''):
        params = {'sprdId': sprdId, 'tradeId': tradeId, 'ordId': ordId, 'beginId': beginId, 'endId': endId,
                  'begin': begin, 'end': end, 'limit': limit}
        return self._request_with_params(GET, SPREAD_GET_TRADES, params)

    # Get Spreads (Public)
    def get_spreads(self, baseCcy='',instId='', sprdId='', state=''):
        params = {'baseCcy': baseCcy, 'instId': instId, 'sprdId': sprdId, 'state': state}
        return self._request_with_params(GET, SPREAD_GET_SPREADS, params)

    # Get order book (Public)
    def get_order_book(self, sprdId='', sz=''):
        params = {'sprdId': sprdId, 'sz': sz}
        return self._request_with_params(GET, SPREAD_GET_ORDER_BOOK, params)

    # Get ticker (Public)
    def get_ticker(self, sprdId=''):
        params = {'sprdId': sprdId}
        return self._request_with_params(GET, SPREAD_GET_TICKER, params)

    # Get public trades (Public)
    def get_public_trades(self, sprdId=''):
        params = {'sprdId': sprdId}
        return self._request_with_params(GET, SPREAD_GET_PUBLIC_TRADES, params)




import asyncio
import json
import logging

from okx.websocket import WsUtils
from okx.websocket.WebSocketFactory import WebSocketFactory

logger = logging.getLogger(__name__)


class WsPrivateAsync:
    def __init__(self, apiKey, passphrase, secretKey, url, useServerTime):
        self.url = url
        self.subscriptions = set()
        self.callback = None
        self.loop = asyncio.get_event_loop()
        self.factory = WebSocketFactory(url)
        self.apiKey = apiKey
        self.passphrase = passphrase
        self.secretKey = secretKey
        self.useServerTime = useServerTime

    async def connect(self):
        self.websocket = await self.factory.connect()

    async def consume(self):
        async for message in self.websocket:
            logger.debug("Received message: {%s}", message)
            if self.callback:
                self.callback(message)

    async def subscribe(self, params: list, callback):
        self.callback = callback

        logRes = await self.login()
        await asyncio.sleep(5)
        if logRes:
            payload = json.dumps({
                "op": "subscribe",
                "args": params
            })
            await self.websocket.send(payload)
        # await self.consume()

    async def login(self):
        loginPayload = WsUtils.initLoginParams(
            useServerTime=self.useServerTime,
            apiKey=self.apiKey,
            passphrase=self.passphrase,
            secretKey=self.secretKey
        )
        await self.websocket.send(loginPayload)
        return True

    async def unsubscribe(self, params: list, callback):
        self.callback = callback
        payload = json.dumps({
            "op": "unsubscribe",
            "args": params
        })
        logger.info(f"unsubscribe: {payload}")
        await self.websocket.send(payload)
        # for param in params:
        #     self.subscriptions.discard(param)

    async def stop(self):
        await self.factory.close()
        self.loop.stop()

    async def start(self):
        logger.info("Connecting to WebSocket...")
        await self.connect()
        self.loop.create_task(self.consume())

    def stop_sync(self):
        self.loop.run_until_complete(self.stop())



from okx.okxclient import OkxClient
from okx.consts import *


class SavingsAPI(OkxClient):
    def __init__(self, api_key='-1', api_secret_key='-1', passphrase='-1', use_server_time=None, flag='1', domain = 'https://www.okx.com',debug = False, proxy=None):
        OkxClient.__init__(self, api_key, api_secret_key, passphrase, use_server_time, flag, domain, debug, proxy)

    # - Get saving balance
    def get_saving_balance(self, ccy=''):
        params = {
            'ccy': ccy
        }
        return self._request_with_params(GET, GET_SAVING_BALANCE, params)

    # - Savings purchase/redemption
    def savings_purchase_redemption(self, ccy='', amt='', side='', rate=''):

        params = {
            'ccy': ccy,
            'amt': amt,
            'side': side,
            'rate': rate
        }
        return self._request_with_params(POST, SAVING_PURCHASE_REDEMPTION, params)

    # - Set lending rate
    def set_lending_rate(self, ccy='', rate=''):
        params = {
            'ccy': ccy,
            'rate': rate
        }
        return self._request_with_params(POST, SET_LENDING_RATE, params)

    # - Get lending history
    def get_lending_history(self, ccy='', after='', before='', limit=''):
        params = {
            'ccy': ccy,
            'after': after,
            'before': before,
            'limit': limit
        }
        return self._request_with_params(GET, GET_LENDING_HISTORY, params)

    # - Get public borrow history (public)
    def get_public_borrow_history(self, ccy='', after='', before='', limit=''):
        params = {
            'ccy': ccy,
            'after': after,
            'before': before,
            'limit': limit
        }
        return self._request_with_params(GET, GET_PUBLIC_BORROW_HISTORY, params)

    # GET / Public borrow info (public)
    def get_public_borrow_info(self, ccy=''):
        params = {
            'ccy': ccy
        }
        return self._request_with_params(GET, GET_PUBLIC_BORROW_INFO, params)





from .BaseClient import OkxBaseClient
from ..constants import *


class FinanceClient(OkxBaseClient):
    def __init__(
        self,
        apikey="",
        apisecret="",
        passphrase="",
        use_server_time=False,
        simulation=False,
        domain=API_URL,
        debug=False,
        proxy=None,
    ):
        OkxBaseClient.__init__(
            self,
            apikey,
            apisecret,
            passphrase,
            use_server_time,
            simulation,
            domain,
            debug,
            proxy,
        )

    def earn_get_offers(self, productId="", protocolType="", ccy=""):
        params = {"productId": productId, "protocolType": protocolType, "ccy": ccy}
        return self._request(GET, FINANCE_STACKING_DEFI_OFFERS, params)

    def earn_purchase(self, productId="", investData=[], term=""):
        params = {
            "productId": productId,
            "investData": investData,
            "tag": BROKER_ID,
        }
        if term != "":
            params["term"] = term
        return self._request(POST, FINANCE_STACKING_DEFI_PURCHASE, params)

    def earn_redeem(self, ordId="", protocolType="", allowEarlyRedeem=""):
        params = {
            "ordId": ordId,
            "protocolType": protocolType,
            "allowEarlyRedeem": allowEarlyRedeem,
        }
        return self._request(POST, FINANCE_STACKING_DEFI_REDEEM, params)

    def earn_cancel(self, ordId="", protocolType=""):
        params = {"ordId": ordId, "protocolType": protocolType}
        return self._request(POST, FINANCE_STACKING_DEFI_CANCEL, params)

    def earn_get_active_orders(self, productId="", protocolType="", ccy="", state=""):
        params = {
            "productId": productId,
            "protocolType": protocolType,
            "ccy": ccy,
            "state": state,
        }
        return self._request(GET, FINANCE_STACKING_DEFI_ORDERS_ACTIVE, params)

    def earn_get_orders_history(
        self, productId="", protocolType="", ccy="", after="", before="", limit=""
    ):
        params = {
            "productId": productId,
            "protocolType": protocolType,
            "ccy": ccy,
            "after": after,
            "before": before,
            "limit": limit,
        }
        return self._request(GET, FINANCE_STACKING_DEFI_ORDERS_HISTORY, params)

    def eth_purchase(self, amt):
        params = {"amt": amt}
        return self._request(POST, FINANCE_STACKING_DEFI_ETH_PURCHASE, params)

    def eth_redeem(self, amt):
        params = {"amt": amt}
        return self._request(POST, FINANCE_STACKING_DEFI_ETH_REDEEM, params)

    def eth_get_balance(self):
        return self._request(GET, FINANCE_STACKING_DEFI_ETH_BALANCE)

    def eth_get_purchase_redeem_history(
        self, type, status="", after="", before="", limit=""
    ):
        params = {
            "type": type,
            "status": status,
            "after": after,
            "before": before,
            "limit": limit,
        }
        return self._request(
            GET, FINANCE_STACKING_DEFI_ETH_PURCHASE_REDEEM_HISTORY, params
        )

    def eth_apy_history(self, days):
        params = {"days": days}
        return self._request(GET, FINANCE_STACKING_DEFI_ETH_APY_HISTORY, params)

    # Get saving balance
    def savings_get_saving_balance(self, ccy=""):
        params = {"ccy": ccy}
        return self._request(GET, FINANCE_SAVINGS_BALANCE, params)

    # Savings purchase/redemption
    def savings_purchase_redemption(self, ccy="", amt="", side="", rate=""):
        params = {"ccy": ccy, "amt": amt, "side": side, "rate": rate}
        return self._request(POST, FINANCE_SAVINGS_PURCHASE_REDEMPT, params)

    # Set lending rate
    def savings_set_lending_rate(self, ccy="", rate=""):
        params = {"ccy": ccy, "rate": rate}
        return self._request(POST, FINANCE_SAVINGS_SET_LENDING_RATE, params)

    # Get lending history
    def savings_get_lending_history(self, ccy="", after="", before="", limit=""):
        params = {"ccy": ccy, "after": after, "before": before, "limit": limit}
        return self._request(GET, FINANCE_SAVINGS_LENDING_HISTORY, params)

    # Get public borrow info (public)
    def savings_get_public_borrow_info(self, ccy=""):
        params = {"ccy": ccy}
        return self._request(GET, FINANCE_SAVINGS_LENDING_RATE_SUMMARY, params)

    # Get public borrow history (public)
    def savings_get_public_borrow_history(self, ccy="", after="", before="", limit=""):
        params = {"ccy": ccy, "after": after, "before": before, "limit": limit}
        return self._request(GET, FINANCE_SAVINGS_LENDING_RATE_HISTORY, params)



from .BaseClient import OkxBaseClient
from ..constants import *


class AccountClient(OkxBaseClient):

    def __init__(
        self,
        apikey="",
        apisecret="",
        passphrase="",
        use_server_time=False,
        simulation=False,
        domain=API_URL,
        debug=False,
        proxy=None,
    ):
        OkxBaseClient.__init__(
            self,
            apikey,
            apisecret,
            passphrase,
            use_server_time,
            simulation,
            domain,
            debug,
            proxy,
        )

    # Get Balance
    def get_account_balance(self, ccy=""):
        params = {}
        if ccy:
            params["ccy"] = ccy
        return self._request(GET, ACCOUNT_BALANCE, params)

    # Get Positions
    def get_positions(self, instType="", instId=""):
        params = {"instType": instType, "instId": instId}
        return self._request(GET, ACCOUNT_POSITIONS, params)

    # Get positions history
    def get_positions_history(
        self,
        instType="",
        instId="",
        mgnMode="",
        type="",
        posId="",
        after="",
        before="",
        limit="",
    ):
        params = {
            "instType": instType,
            "instId": instId,
            "mgnMode": mgnMode,
            "type": type,
            "posId": posId,
            "after": after,
            "before": before,
            "limit": limit,
        }
        return self._request(GET, ACCOUNT_POSITIONS_HISTORY, params)

    # Get account and position risk
    def get_position_risk(self, instType=""):
        params = {}
        if instType:
            params["instType"] = instType
        return self._request(GET, ACCOUNT_ACCOUNT_POSITION_RISK, params)

    # Get Bills Details (recent 7 days)
    def get_account_bills(
        self,
        instType="",
        ccy="",
        mgnMode="",
        ctType="",
        type="",
        subType="",
        after="",
        before="",
        limit="",
    ):
        params = {
            "instType": instType,
            "ccy": ccy,
            "mgnMode": mgnMode,
            "ctType": ctType,
            "type": type,
            "subType": subType,
            "after": after,
            "before": before,
            "limit": limit,
        }
        return self._request(GET, ACCOUNT_BILLS, params)

    # Get Bills Details (recent 3 months)
    def get_account_bills_archive(
        self,
        instType="",
        ccy="",
        mgnMode="",
        ctType="",
        type="",
        subType="",
        after="",
        before="",
        limit="",
    ):
        params = {
            "instType": instType,
            "ccy": ccy,
            "mgnMode": mgnMode,
            "ctType": ctType,
            "type": type,
            "subType": subType,
            "after": after,
            "before": before,
            "limit": limit,
        }
        return self._request(GET, ACCOUNT_BILLS_ARCHIVE, params)

    # Get Account Configuration
    def get_account_config(self):
        return self._request(GET, ACCOUNT_CONFIG)

    # Set position mode
    def set_position_mode(self, posMode):
        params = {"posMode": posMode}
        return self._request(POST, ACCOUNT_SET_POSITION_MODE, params)

    # Set leverage
    def set_leverage(self, lever, mgnMode, instId="", ccy="", posSide=""):
        params = {
            "lever": lever,
            "mgnMode": mgnMode,
            "instId": instId,
            "ccy": ccy,
            "posSide": posSide,
        }
        return self._request(POST, ACCOUNT_SET_LEVERAGE, params)

    # Get maximum buy/sell amount or open amount
    def get_max_order_size(self, instId, tdMode, ccy="", px=""):
        params = {"instId": instId, "tdMode": tdMode, "ccy": ccy, "px": px}
        return self._request(GET, ACCOUNT_MAX_SIZE, params)

    # Get maximum available tradable amount
    def get_max_avail_size(
        self, instId, tdMode, ccy="", reduceOnly="", unSpotOffset="", quickMgnType=""
    ):
        params = {
            "instId": instId,
            "tdMode": tdMode,
            "ccy": ccy,
            "reduceOnly": reduceOnly,
            "unSpotOffset": unSpotOffset,
            "quickMgnType": quickMgnType,
        }
        return self._request(GET, ACCOUNT_MAX_AVAIL_SIZE, params)

    # Increase/decrease margin
    def adjust_margin(self, instId, posSide, type, amt, loanTrans=""):
        params = {
            "instId": instId,
            "posSide": posSide,
            "type": type,
            "amt": amt,
            "loanTrans": loanTrans,
        }
        return self._request(POST, ACCOUNT_POSITION_MARGIN_BALANCE, params)

    # Get leverage
    def get_leverage(self, instId, mgnMode):
        params = {"instId": instId, "mgnMode": mgnMode}
        return self._request(GET, ACCOUNT_LEVERAGE_INFO, params)

    # Get leverage estimated info
    def get_leverage_estimated_info(
        self, instType, mgnMode, lever, instId="", ccy="", posSide=""
    ):
        params = {
            "instType": instType,
            "mgnMode": mgnMode,
            "lever": lever,
            "instId": instId,
            "ccy": ccy,
            "posSide": posSide,
        }
        return self._request(GET, ACCOUNT_ADJUST_LEVERAGE_INFO, params)

    # Get the maximum loan of instrument
    def get_max_loan(self, instId, mgnMode, mgnCcy):
        params = {"instId": instId, "mgnMode": mgnMode, "mgnCcy": mgnCcy}
        return self._request(GET, ACCOUNT_MAX_LOAN, params)

    # Get Fee Rates
    def get_fee_rates(self, instType, instId="", uly="", category="", instFamily=""):
        params = {
            "instType": instType,
            "instId": instId,
            "uly": uly,
            "category": category,
            "instFamily": instFamily,
        }
        return self._request(GET, ACCOUNT_TRADE_FEE, params)

    # Get interest accrued data
    def get_interest_accrued(
        self, instId="", ccy="", mgnMode="", after="", before="", limit=""
    ):
        params = {
            "instId": instId,
            "ccy": ccy,
            "mgnMode": mgnMode,
            "after": after,
            "before": before,
            "limit": limit,
        }
        return self._request(GET, ACCOUNT_INTEREST_ACCRUED, params)

    # Get interest rate
    def get_interest_rate(self, ccy=""):
        params = {"ccy": ccy}
        return self._request(GET, ACCOUNT_INTEREST_RATE, params)

    # Set greeks (PA/BS)
    def set_greeks(self, greeksType):
        params = {"greeksType": greeksType}
        return self._request(POST, ACCOUNT_SET_GREEKS, params)

    # Isolated margin trading settings
    def set_isolated_mode(self, isoMode, type):
        params = {"isoMode": isoMode, "type": type}
        return self._request(POST, ACCOUNT_ISOLATED_MODE, params)

    # Get maximum withdrawals
    def get_max_withdrawal(self, ccy=""):
        params = {"ccy": ccy}
        return self._request(GET, ACCOUNT_MAX_WITHDRAWAL, params)

    # Get account risk state
    def get_account_position_risk(self):
        return self._request(GET, ACCOUNT_RISK_STATE)

    # Manual borrow and repay in Quick Margin Mode
    def quick_margin_borrow_repay(self, instId, ccy, side, amt):
        params = {"instId": instId, "ccy": ccy, "side": side, "amt": amt}
        return self._request(POST, ACCOUNT_QUICK_MARGIN_BORROW_REPAY, params)

    # Get borrow and repay history in Quick Margin Mode
    def get_quick_margin_borrow_repay_history(
        self,
        instId="",
        ccy="",
        side="",
        after="",
        before="",
        begin="",
        end="",
        limit="",
    ):
        params = {
            "instId": instId,
            "ccy": ccy,
            "side": side,
            "after": after,
            "before": before,
            "begin": begin,
            "end": end,
            "limit": limit,
        }
        return self._request(GET, ACCOUNT_QUICK_MARGIN_BORROW_REPAY_HISTORY, params)

    # VIP loans borrow and repay
    def borrow_repay(self, ccy="", side="", amt="", ordId=""):
        params = {"ccy": ccy, "side": side, "amt": amt, "ordId": ordId}
        return self._request(POST, ACCOUNT_BORROW_REPAY, params)

    # Get borrow and repay history for VIP loans
    def get_borrow_repay_history(self, ccy="", after="", before="", limit=""):
        params = {"ccy": ccy, "after": after, "before": before, "limit": limit}
        return self._request(GET, ACCOUNT_BORROW_REPAY_HISTORY, params)

    # Get VIP interest accrued data
    def get_VIP_interest_accrued_data(
        self, ccy="", ordId="", after="", before="", limit=""
    ):
        params = {
            "ccy": ccy,
            "ordId": ordId,
            "after": after,
            "before": before,
            "limit": limit,
        }
        return self._request(GET, ACCOUNT_VIP_INTEREST_ACCRUED, params)

    # Get VIP interest deducted data
    def get_vip_interest_deducted_data(
        self, ccy="", ordId="", after="", before="", limit=""
    ):
        params = {
            "ccy": ccy,
            "ordId": ordId,
            "after": after,
            "before": before,
            "limit": limit,
        }
        return self._request(GET, ACCOUNT_VIP_INTEREST_DEDUCTED, params)

    # Get VIP loan order list
    def get_vip_loan_order_list(
        self, ordId="", state="", ccy="", after="", before="", limit=""
    ):
        params = {
            "ordId": ordId,
            "state": state,
            "ccy": ccy,
            "after": after,
            "before": before,
            "limit": limit,
        }
        return self._request(GET, ACCOUNT_VIP_LOAN_ORDER_LIST, params)

    # Get VIP loan order detail
    def get_vip_loan_order_detail(
        self, ccy="", ordId="", after="", before="", limit=""
    ):
        params = {
            "ccy": ccy,
            "ordId": ordId,
            "after": after,
            "before": before,
            "limit": limit,
        }
        return self._request(GET, ACCOUNT_VIP_LOAN_ORDER_DETAIL, params)

    # Get borrow interest and limit
    def get_interest_limits(self, type="", ccy=""):
        params = {"type": type, "ccy": ccy}
        return self._request(GET, ACCOUNT_INTEREST_LIMITS, params)

    # Position builder
    def simulated_margin(
        self, instType="", inclRealPos=True, spotOffsetType="", simPos=[]
    ):
        params = {
            "instType": instType,
            "inclRealPos": inclRealPos,
            "spotOffsetType": spotOffsetType,
            "simPos": simPos,
        }
        return self._request(POST, ACCOUNT_SIMULATED_MARGIN, params)

    # Position builder (new)
    def position_builder(self, inclRealPosAndEq=True, spotOffsetType="", simPos=[]):
        params = {
            "inclRealPosAndEq": inclRealPosAndEq,
            "spotOffsetType": spotOffsetType,
            "simPos": simPos,
        }
        return self._request(POST, ACCOUNT_POSITION_BUILDER, params)

    # Get  Greeks
    def get_greeks(self, ccy=""):
        params = {"ccy": ccy}
        return self._request(GET, ACCOUNT_GREEKS, params)

    # Get PM position limitation
    def get_account_position_tiers(self, instType="", uly="", instFamily=""):
        params = {"instType": instType, "uly": uly, "instFamily": instFamily}
        return self._request(GET, ACCOUNT_POSITION_TIERS, params)

    # Set risk offset type
    def set_risk_offset_typel(self, type=""):
        params = {"type": type}
        return self._request(POST, ACCOUNT_SET_RISK_OFFSET_TYPE, params)

    # Activate option
    def activate_option(self):
        return self._request(POST, ACCOUNT_ACTIVATE_OPTION)

    # Set auto loan
    def set_auto_loan(self, autoLoan=""):
        params = {"autoLoan": autoLoan}
        return self._request(POST, ACCOUNT_SET_AUTO_LOAN, params)

    # Set account mode
    def set_account_mode(self, acctLv):
        params = {"acctLv": acctLv}
        return self._request(POST, ACCOUNT_SET_ACCOUNT_LEVEL, params)

    # Reset MMP Status
    def reset_mmp_status(self, instFamily, instType=""):
        params = {"instFamily": instFamily, "instType": instType}
        return self._request(POST, ACCOUNT_MMP_RESET, params)

    # Set MMP
    def set_mmp_config(self, instFamily, timeInterval, frozenInterval, qtyLimit):
        params = {
            "instFamily": instFamily,
            "timeInterval": timeInterval,
            "frozenInterval": frozenInterval,
            "qtyLimit": qtyLimit,
        }
        return self._request(POST, ACCOUNT_SET_MMP_CONFIG, params)

    # GET MMP Config
    def get_mmp_config(self, instFamily=""):
        params = {"instFamily": instFamily}
        return self._request(GET, ACCOUNT_GET_MMP_CONFIG, params)

    # Get the invitee's detail
    def get_the_invitee_details(self, uid=""):
        params = {"uid": uid}
        return self._request(GET, AFFILIATE_INVITEE_DETAIL, params)

    # Get the user's affiliate rebate information
    def get_the_user_affiliate_rebate_information(self, apiKey=""):
        params = {"apiKey": apiKey}
        return self._request(GET, USERS_PARTNER_IF_REBATE, params)



from .BaseClient import OkxBaseClient
from ..constants import *


class FundingClient(OkxBaseClient):

    def __init__(
        self,
        apikey="",
        apisecret="",
        passphrase="",
        use_server_time=False,
        simulation=False,
        domain=API_URL,
        debug=False,
        proxy=None,
    ):
        OkxBaseClient.__init__(
            self,
            apikey,
            apisecret,
            passphrase,
            use_server_time,
            simulation,
            domain,
            debug,
            proxy,
        )

    # Get Currencies
    def get_currencies(self, ccy=""):
        params = {"ccy": ccy}
        return self._request(GET, ASSET_CURRENCIES, params)

    # Get Balance
    def get_balances(self, ccy=""):
        params = {"ccy": ccy}
        return self._request(GET, ASSET_BALANCES, params)

    # Get non-tradable assets
    def get_non_tradable_assets(self, ccy=""):
        params = {"ccy": ccy}
        return self._request(GET, ASSET_NON_TRADABLE_ASSETS, params)

    # Get account asset valuation
    def get_asset_valuation(self, ccy=""):
        params = {"ccy": ccy}
        return self._request(GET, ASSET_ASSET_VALUATION, params)

    # Funds transfer
    def funds_transfer(
        self,
        ccy,
        amt,
        from_,
        to,
        type="0",
        subAcct="",
        instId="",
        toInstId="",
        loanTrans="",
    ):
        params = {
            "ccy": ccy,
            "amt": amt,
            "from": from_,
            "to": to,
            "type": type,
            "subAcct": subAcct,
            "instId": instId,
            "toInstId": toInstId,
            "loanTrans": loanTrans,
        }
        return self._request(POST, ASSET_TRANSFER, params)

    # Get funds transfer state
    def transfer_state(self, transId, type=""):
        params = {"transId": transId, "type": type}
        return self._request(GET, ASSET_TRANSFER_STATE, params)

    # Asset bills details
    def get_bills(self, ccy="", type="", after="", before="", limit=""):
        params = {
            "ccy": ccy,
            "type": type,
            "after": after,
            "before": before,
            "limit": limit,
        }
        return self._request(GET, ASSET_BILLS, params)

    # Lightning deposits
    def get_deposit_lightning(self, ccy, amt, to=""):
        params = {"ccy": ccy, "amt": amt}
        if to:
            params = {"to": to}
        return self._request(GET, ASSET_DEPOSIT_LIGHTNING, params)

    # Get Deposit Address
    def get_deposit_address(self, ccy):
        params = {"ccy": ccy}
        return self._request(GET, ASSET_DEPOSIT_ADDRESS, params)

    # Get Deposit History
    def get_deposit_history(
        self,
        ccy="",
        state="",
        after="",
        before="",
        limit="",
        txId="",
        depId="",
        fromWdId="",
    ):
        params = {
            "ccy": ccy,
            "state": state,
            "after": after,
            "before": before,
            "limit": limit,
            "txId": txId,
            "depId": depId,
            "fromWdId": fromWdId,
        }
        return self._request(GET, ASSET_DEPOSIT_HISTORY, params)

    # Withdrawal
    def withdrawal(
        self, ccy, amt, dest, toAddr, fee, chain="", areaCode="", clientId=""
    ):
        params = {
            "ccy": ccy,
            "amt": amt,
            "dest": dest,
            "toAddr": toAddr,
            "fee": fee,
            "chain": chain,
            "areaCode": areaCode,
            "clientId": clientId,
        }
        return self._request(POST, ASSET_WITHDRAWAL, params)

    # Withdrawal Lightning
    def withdrawal_lightning(self, ccy, invoice, memo=""):
        params = {"ccy": ccy, "invoice": invoice, "memo": memo}
        return self._request(POST, ASSET_WITHDRAWAL_LIGHTNING, params)

    # Cancel withdrawal
    def cancel_withdrawal(self, wdId=""):
        params = {"wdId": wdId}
        return self._request(POST, ASSET_CANCEL_WITHDRAWAL, params)

    # Get Withdrawal History
    def get_withdrawal_history(
        self, ccy="", wdId="", state="", after="", before="", limit="", txId=""
    ):
        params = {
            "ccy": ccy,
            "wdId": wdId,
            "state": state,
            "after": after,
            "before": before,
            "limit": limit,
            "txId": txId,
        }
        return self._request(GET, ASSET_WITHDRAWAL_HISTORY, params)

    # Get deposit withdraw status
    def get_deposit_withdraw_status(self, wdId="", txId="", ccy="", to="", chain=""):
        params = {"wdId": wdId, "txId": txId, "ccy": ccy, "to": to, "chain": chain}
        return self._request(GET, ASSET_DEPOSIT_WITHDRAW_STATUS, params)

    # Small assets convert
    def convert_dust_assets(self, ccy=[]):
        params = {"ccy": ccy}
        return self._request(POST, ASSET_CONVERT_DUST_ASSETS, params)

    # Get exchange list (public)
    def get_exchange_list(self):
        return self._request(GET, ASSET_EXCHANGE_LIST)

    # Apply for monthly statement
    def apply_monthly_statement(self, month):
        params = {"month": month}
        return self._request(POST, ASSET_MONTHLY_STATEMENT_APPLY, params)

    # Apply for monthly statement
    def get_monthly_statement(self, month):
        params = {"month": month}
        return self._request(GET, ASSET_MONTHLY_STATEMENT_RETRIEVE, params)

    def get_convert_currencies(self):
        params = {}
        return self._request(GET, ASSET_CONVERT_CURRENCIES, params)

    def get_convert_currency_pair(self, fromCcy="", toCcy=""):
        params = {"fromCcy": fromCcy, "toCcy": toCcy}
        return self._request(GET, ASSET_CONVERT_CURRENCY_PAIR, params)

    def estimate_quote(
        self, baseCcy="", quoteCcy="", side="", rfqSz="", rfqSzCcy="", clQReqId=""
    ):
        params = {
            "baseCcy": baseCcy,
            "quoteCcy": quoteCcy,
            "side": side,
            "rfqSz": rfqSz,
            "rfqSzCcy": rfqSzCcy,
            "clQReqId": clQReqId,
            "tag": BROKER_ID,
        }
        return self._request(POST, ASSET_CONVERT_ESTIMATE_QUOTE, params)

    def convert_trade(
        self, quoteId="", baseCcy="", quoteCcy="", side="", sz="", szCcy="", clTReqId=""
    ):
        params = {
            "quoteId": quoteId,
            "baseCcy": baseCcy,
            "quoteCcy": quoteCcy,
            "side": side,
            "sz": sz,
            "szCcy": szCcy,
            "clTReqId": clTReqId,
            "tag": BROKER_ID,
        }
        return self._request(POST, ASSET_CONVERT_TRADE, params)

    def get_convert_history(self, after="", before="", limit="", tag=""):
        params = {"after": after, "before": before, "limit": limit, "tag": tag}
        return self._request(GET, ASSET_CONVERT_HISTORY, params)
